@@ -2,6 +2,7 @@ from __future__ import division  # confidence high
 from __future__ import with_statement
 
 import itertools
+import sys
 import warnings
 
 import numpy as np
@@ -1883,6 +1884,28 @@ class TestHeaderFunctions(PyfitsTestCase):
             c.verify('fix')
         assert_equal(str(c), _pad('HIERARCH ESO DET CHIP PXSPACE = 5E6'))
 
+    def test_assign_inf_nan(self):
+        """
+        Regression test for https://github.com/spacetelescope/PyFITS/issues/11
+
+        For the time being it should not be possible to assign the floating
+        point values inf or nan to a header value, since this is not defined by
+        the FITS standard.
+        """
+
+        h = pyfits.Header()
+
+        # There is an obscure cross-platform issue that prevents things like
+        # float('nan') on Windows on older versions of Python; hence it is
+        # unlikely to come up in practice
+        if not (sys.platform.startswith('win32') and
+                sys.version_info[:2] < (2, 6)):
+           assert_raises(ValueError, h.set, 'TEST', float('nan'))
+           assert_raises(ValueError, h.set, 'TEST', float('inf'))
+
+        assert_raises(ValueError, h.set, 'TEST', np.nan)
+        assert_raises(ValueError, h.set, 'TEST', np.inf)
+
 
 class TestRecordValuedKeywordCards(PyfitsTestCase):
     """
